@@ -24,16 +24,44 @@ from test_build_html import ENV_WARNINGS
 
 
 LATEX_WARNINGS = ENV_WARNINGS + """\
-%(root)s/markup.txt:158: WARNING: unknown option: &option
+%(root)s/markup.txt:160: WARNING: unknown option: &option
 %(root)s/footnote.txt:60: WARNING: citation not found: missing
 %(root)s/images.txt:20: WARNING: no matching candidate for image URI u'foo.\\*'
-%(root)s/markup.txt:269: WARNING: Could not parse literal_block as "c". highlighting skipped.
+%(root)s/markup.txt:271: WARNING: Could not lex literal_block as "c". Highlighting skipped.
 """
 
 if PY3:
     LATEX_WARNINGS = remove_unicode_literals(LATEX_WARNINGS)
 
 
+def run_latex(outdir):
+    """Run pdflatex, xelatex, and lualatex in the outdir"""
+    cwd = os.getcwd()
+    os.chdir(outdir)
+    try:
+        latexes = ('pdflatex', 'xelatex', 'lualatex')
+        available_latexes = len(latexes)
+        for latex in latexes:
+            try:
+                os.mkdir(latex)
+                p = Popen([latex, '--interaction=nonstopmode',
+                        '-output-directory=%s' % latex, 'SphinxTests.tex'],
+                        stdout=PIPE, stderr=PIPE)
+            except OSError:  # most likely the latex executable was not found
+                available_latexes -= 1
+            else:
+                stdout, stderr = p.communicate()
+                if p.returncode != 0:
+                    print(stdout)
+                    print(stderr)
+                    assert False, '%s exited with return code %s' % (
+                        latex, p.returncode)
+    finally:
+        os.chdir(cwd)
+
+    if available_latexes == 0: # no latex is available, skip the test
+        raise SkipTest
+
 @with_app(buildername='latex', freshenv=True)  # use freshenv to check warnings
 def test_latex(app, status, warning):
     LaTeXTranslator.ignore_missing_images = True
@@ -75,22 +103,7 @@ def test_latex(app, status, warning):
                            'seem to be installed' % filename)
 
     # now, try to run latex over it
-    cwd = os.getcwd()
-    os.chdir(app.outdir)
-    try:
-        try:
-            p = Popen(['pdflatex', '--interaction=nonstopmode',
-                       'SphinxTests.tex'], stdout=PIPE, stderr=PIPE)
-        except OSError:
-            raise SkipTest  # most likely pdflatex was not found
-        else:
-            stdout, stderr = p.communicate()
-            if p.returncode != 0:
-                print(stdout)
-                print(stderr)
-                assert False, 'latex exited with return code %s' % p.returncode
-    finally:
-        os.chdir(cwd)
+    run_latex(app.outdir)
 
 
 @with_app(buildername='latex', freshenv=True,  # use freshenv to check warnings
@@ -139,23 +152,7 @@ def test_latex_howto(app, status, warning):
                            'seem to be installed' % filename)
 
     # now, try to run latex over it
-    cwd = os.getcwd()
-    os.chdir(app.outdir)
-    try:
-        try:
-            p = Popen(['pdflatex', '--interaction=nonstopmode',
-                       'SphinxTests.tex'], stdout=PIPE, stderr=PIPE)
-        except OSError:
-            raise SkipTest  # most likely pdflatex was not found
-        else:
-            stdout, stderr = p.communicate()
-            if p.returncode != 0:
-                print(stdout)
-                print(stderr)
-                app.cleanup()
-                assert False, 'latex exited with return code %s' % p.returncode
-    finally:
-        os.chdir(cwd)
+    run_latex(app.outdir)
 
 
 @with_app(buildername='latex', testroot='numfig',
@@ -267,6 +264,25 @@ def test_numref_with_language_ja(app, status, warning):
     assert '\\hyperref[baz:code22]{Code-\\ref{baz:code22}}' in result
 
 
+@with_app(buildername='latex', testroot='numfig',
+          confoverrides={'numfig': True, 'language': 'ru', 'latex_elements': {'babel': ''}})
+def test_numref_on_bable_disabled(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\renewcommand{\\figurename}{Fig. }' in result
+    assert '\\renewcommand{\\tablename}{Table }' in result
+    assert '\\SetupFloatingEnvironment{literal-block}{name=Listing }' in result
+    assert '\\hyperref[index:fig1]{Fig. \\ref{index:fig1}}' in result
+    assert '\\hyperref[baz:fig22]{Figure\\ref{baz:fig22}}' in result
+    assert '\\hyperref[index:table-1]{Table \\ref{index:table-1}}' in result
+    assert '\\hyperref[baz:table22]{Table:\\ref{baz:table22}}' in result
+    assert '\\hyperref[index:code-1]{Listing \\ref{index:code-1}}' in result
+    assert '\\hyperref[baz:code22]{Code-\\ref{baz:code22}}' in result
+
+
 @with_app(buildername='latex')
 def test_latex_add_latex_package(app, status, warning):
     app.add_latex_package('foo')
@@ -277,26 +293,99 @@ def test_latex_add_latex_package(app, status, warning):
     assert '\\usepackage[baz]{bar}' in result
 
 
-@with_app(buildername='latex', testroot='contentsname')
-def test_contentsname(app, status, warning):
+@with_app(buildername='latex', testroot='latex-babel')
+def test_babel_with_no_language_settings(app, status, warning):
     app.builder.build_all()
     result = (app.outdir / 'Python.tex').text(encoding='utf8')
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert ('\\addto\\captionsenglish{\\renewcommand{\\contentsname}{Table of content}}'
+    assert '\\documentclass[letterpaper,10pt,english]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' in result
+    assert '\\usepackage{times}' in result
+    assert '\\usepackage[Bjarne]{fncychap}' in result
+    assert ('\\addto\\captionsenglish{\\renewcommand{\\contentsname}{Table of content}}\n'
             in result)
+    assert '\\addto\\captionsenglish{\\renewcommand{\\figurename}{Fig. }}\n' in result
+    assert '\\addto\\captionsenglish{\\renewcommand{\\tablename}{Table. }}\n' in result
+    assert '\\addto\\extrasenglish{\\def\\pageautorefname{page}}\n' in result
 
 
-@with_app(buildername='latex', testroot='contentsname',
-          confoverrides={'language': 'ja'})
-def test_contentsname_with_language_ja(app, status, warning):
+@with_app(buildername='latex', testroot='latex-babel',
+          confoverrides={'language': 'de'})
+def test_babel_with_language_de(app, status, warning):
     app.builder.build_all()
     result = (app.outdir / 'Python.tex').text(encoding='utf8')
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert '\\renewcommand{\\contentsname}{Table of content}' in result
+    assert '\\documentclass[letterpaper,10pt,ngerman]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' in result
+    assert '\\usepackage{times}' in result
+    assert '\\usepackage[Sonny]{fncychap}' in result
+    assert ('\\addto\\captionsngerman{\\renewcommand{\\contentsname}{Table of content}}\n'
+            in result)
+    assert '\\addto\\captionsngerman{\\renewcommand{\\figurename}{Fig. }}\n' in result
+    assert '\\addto\\captionsngerman{\\renewcommand{\\tablename}{Table. }}\n' in result
+    assert '\\addto\\extrasngerman{\\def\\pageautorefname{page}}\n' in result
+
+
+@with_app(buildername='latex', testroot='latex-babel',
+          confoverrides={'language': 'ru'})
+def test_babel_with_language_ru(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\documentclass[letterpaper,10pt,russian]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' in result
+    assert '\\usepackage{times}' not in result
+    assert '\\usepackage[Sonny]{fncychap}' in result
+    assert ('\\addto\\captionsrussian{\\renewcommand{\\contentsname}{Table of content}}\n'
+            in result)
+    assert '\\addto\\captionsrussian{\\renewcommand{\\figurename}{Fig. }}\n' in result
+    assert '\\addto\\captionsrussian{\\renewcommand{\\tablename}{Table. }}\n' in result
+    assert '\\addto\\extrasrussian{\\def\\pageautorefname{page}}\n' in result
+
+
+@with_app(buildername='latex', testroot='latex-babel',
+          confoverrides={'language': 'ja'})
+def test_babel_with_language_ja(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\documentclass[letterpaper,10pt,dvipdfmx]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' not in result
+    assert '\\usepackage{times}' in result
+    assert '\\usepackage[Sonny]{fncychap}' not in result
+    assert '\\renewcommand{\\contentsname}{Table of content}\n' in result
+    assert '\\renewcommand{\\figurename}{Fig. }\n' in result
+    assert '\\renewcommand{\\tablename}{Table. }\n' in result
+    assert '\\def\\pageautorefname{page}\n' in result
+
+
+@with_app(buildername='latex', testroot='latex-babel',
+          confoverrides={'language': 'unknown'})
+def test_babel_with_unknown_language(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\documentclass[letterpaper,10pt,english]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' in result
+    assert '\\usepackage{times}' in result
+    assert '\\usepackage[Sonny]{fncychap}' in result
+    assert ('\\addto\\captionsenglish{\\renewcommand{\\contentsname}{Table of content}}\n'
+            in result)
+    assert '\\addto\\captionsenglish{\\renewcommand{\\figurename}{Fig. }}\n' in result
+    assert '\\addto\\captionsenglish{\\renewcommand{\\tablename}{Table. }}\n' in result
+    assert '\\addto\\extrasenglish{\\def\\pageautorefname{page}}\n' in result
+
+    assert "WARNING: no Babel option known for language 'unknown'" in warning.getvalue()
 
 
 @with_app(buildername='latex')
@@ -306,18 +395,18 @@ def test_footnote(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert '\\footnote[1]{\nnumbered\n}' in result
-    assert '\\footnote[2]{\nauto numbered\n}' in result
-    assert '\\footnote[3]{\nnamed\n}' in result
-    assert '{\\hyperref[footnote:bar]{\\emph{{[}bar{]}}}}' in result
+    assert '\\footnote[1]{\sphinxAtStartFootnote%\nnumbered\n}' in result
+    assert '\\footnote[2]{\sphinxAtStartFootnote%\nauto numbered\n}' in result
+    assert '\\footnote[3]{\sphinxAtStartFootnote%\nnamed\n}' in result
+    assert '{\\hyperref[footnote:bar]{\\crossref{{[}bar{]}}}}' in result
     assert '\\bibitem[bar]{bar}{\\phantomsection\\label{footnote:bar} ' in result
     assert '\\bibitem[bar]{bar}{\\phantomsection\\label{footnote:bar} \ncite' in result
     assert '\\bibitem[bar]{bar}{\\phantomsection\\label{footnote:bar} \ncite\n}' in result
     assert '\\capstart\\caption{Table caption \\protect\\footnotemark[4]}' in result
     assert 'name \\protect\\footnotemark[5]' in result
     assert ('\\end{threeparttable}\n\n'
-            '\\footnotetext[4]{\nfootnotes in table caption\n}'
-            '\\footnotetext[5]{\nfootnotes in table\n}' in result)
+            '\\footnotetext[4]{\sphinxAtStartFootnote%\nfootnotes in table caption\n}'
+            '\\footnotetext[5]{\sphinxAtStartFootnote%\nfootnotes in table\n}' in result)
 
 
 @with_app(buildername='latex', testroot='footnotes')
@@ -328,21 +417,24 @@ def test_reference_in_caption(app, status, warning):
     print(status.getvalue())
     print(warning.getvalue())
     assert ('\\caption{This is the figure caption with a reference to \\label{index:id2}'
-            '{\\hyperref[index:authoryear]{\\emph{{[}AuthorYear{]}}}}.}' in result)
+            '{\\hyperref[index:authoryear]{\\crossref{{[}AuthorYear{]}}}}.}' in result)
     assert '\\chapter{The section with a reference to {[}AuthorYear{]}}' in result
     assert '\\caption{The table title with a reference to {[}AuthorYear{]}}' in result
     assert '\\paragraph{The rubric title with a reference to {[}AuthorYear{]}}' in result
     assert ('\\chapter{The section with a reference to \\protect\\footnotemark[4]}\n'
             '\\label{index:the-section-with-a-reference-to}'
-            '\\footnotetext[4]{\nFootnote in section\n}' in result)
+            '\\footnotetext[4]{\sphinxAtStartFootnote%\nFootnote in section\n}' in result)
     assert ('\\caption{This is the figure caption with a footnote to '
-            '\\protect\\footnotemark[6].}\end{figure}\n'
-            '\\footnotetext[6]{\nFootnote in caption\n}')in result
+            '\\protect\\footnotemark[6].}\label{index:id23}\end{figure}\n'
+            '\\footnotetext[6]{\sphinxAtStartFootnote%\nFootnote in caption\n}')in result
     assert ('\\caption{footnote \\protect\\footnotemark[7] '
             'in caption of normal table}') in result
-    assert '\\end{threeparttable}\n\n\\footnotetext[7]{\nFoot note in table\n}' in result
-    assert '\\caption{footnote \\protect\\footnotemark[8] in caption of longtable}' in result
-    assert '\end{longtable}\n\n\\footnotetext[8]{\nFoot note in longtable\n}' in result
+    assert ('\\end{threeparttable}\n\n\\footnotetext[7]{\sphinxAtStartFootnote%\n'
+            'Foot note in table\n}' in result)
+    assert ('\\caption{footnote \\protect\\footnotemark[8] in caption of longtable}'
+            in result)
+    assert ('\end{longtable}\n\n\\footnotetext[8]{\sphinxAtStartFootnote%\n'
+            'Foot note in longtable\n}' in result)
 
 
 @with_app(buildername='latex', testroot='footnotes',
@@ -353,20 +445,31 @@ def test_latex_show_urls_is_inline(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert 'First footnote: \\footnote[2]{\nFirst\n}' in result
-    assert 'Second footnote: \\footnote[1]{\nSecond\n}' in result
+    assert ('Same footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in bar\n} in bar.rst' in result)
+    assert ('Auto footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in baz\n} in baz.rst' in result)
+    assert ('\\phantomsection\\label{index:id26}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to\\string-authoryear]'
+            '{\\crossref{The section with a reference to \\phantomsection\\label{index:id1}'
+            '{\\hyperref[index:authoryear]{\\crossref{{[}AuthorYear{]}}}}}}}' in result)
+    assert ('\\phantomsection\\label{index:id27}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to]'
+            '{\\crossref{The section with a reference to }}}' in result)
+    assert 'First footnote: \\footnote[2]{\sphinxAtStartFootnote%\nFirst\n}' in result
+    assert 'Second footnote: \\footnote[1]{\sphinxAtStartFootnote%\nSecond\n}' in result
     assert '\\href{http://sphinx-doc.org/}{Sphinx} (http://sphinx-doc.org/)' in result
-    assert 'Third footnote: \\footnote[3]{\nThird\n}' in result
+    assert 'Third footnote: \\footnote[3]{\sphinxAtStartFootnote%\nThird\n}' in result
     assert ('\\href{http://sphinx-doc.org/~test/}{URL including tilde} '
             '(http://sphinx-doc.org/\\textasciitilde{}test/)' in result)
     assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term} (http://sphinx-doc.org/)}] '
             '\\leavevmode\nDescription' in result)
     assert ('\\item[{Footnote in term \\protect\\footnotemark[5]}] '
-            '\\leavevmode\\footnotetext[5]{\nFootnote in term\n}\nDescription' in result)
+            '\\leavevmode\\footnotetext[5]{\sphinxAtStartFootnote%\n'
+            'Footnote in term\n}\nDescription' in result)
     assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist} '
             '(http://sphinx-doc.org/)}] \\leavevmode\nDescription' in result)
-    assert ('\\href{https://github.com/sphinx-doc/sphinx}'
-            '{https://github.com/sphinx-doc/sphinx}\n' in result)
+    assert ('\\url{https://github.com/sphinx-doc/sphinx}\n' in result)
     assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
             '{sphinx-dev@googlegroups.com}' in result)
 
@@ -379,22 +482,36 @@ def test_latex_show_urls_is_footnote(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert 'First footnote: \\footnote[2]{\nFirst\n}' in result
-    assert 'Second footnote: \\footnote[1]{\nSecond\n}' in result
+    assert ('Same footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in bar\n} in bar.rst' in result)
+    assert ('Auto footnote number \\footnote[2]{\sphinxAtStartFootnote%\n'
+            'footnote in baz\n} in baz.rst' in result)
+    assert ('\\phantomsection\\label{index:id26}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to\\string-authoryear]'
+            '{\\crossref{The section with a reference to \\phantomsection\\label{index:id1}'
+            '{\\hyperref[index:authoryear]{\\crossref{{[}AuthorYear{]}}}}}}}' in result)
+    assert ('\\phantomsection\\label{index:id27}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to]'
+            '{\\crossref{The section with a reference to }}}' in result)
+    assert 'First footnote: \\footnote[3]{\sphinxAtStartFootnote%\nFirst\n}' in result
+    assert 'Second footnote: \\footnote[1]{\sphinxAtStartFootnote%\nSecond\n}' in result
     assert ('\\href{http://sphinx-doc.org/}{Sphinx}'
-            '\\footnote[3]{\nhttp://sphinx-doc.org/\n}' in result)
-    assert 'Third footnote: \\footnote[5]{\nThird\n}' in result
+            '\\footnote[4]{\sphinxAtStartFootnote%\nhttp://sphinx-doc.org/\n}' in result)
+    assert 'Third footnote: \\footnote[6]{\sphinxAtStartFootnote%\nThird\n}' in result
     assert ('\\href{http://sphinx-doc.org/~test/}{URL including tilde}'
-            '\\footnote[4]{\nhttp://sphinx-doc.org/\\textasciitilde{}test/\n}' in result)
-    assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term}\\protect\\footnotemark[7]}] '
-            '\\leavevmode\\footnotetext[7]{\nhttp://sphinx-doc.org/\n}\nDescription' in result)
-    assert ('\\item[{Footnote in term \\protect\\footnotemark[9]}] '
-            '\\leavevmode\\footnotetext[9]{\nFootnote in term\n}\nDescription' in result)
+            '\\footnote[5]{\sphinxAtStartFootnote%\n'
+            'http://sphinx-doc.org/\\textasciitilde{}test/\n}' in result)
+    assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term}\\protect\\footnotemark[8]}] '
+            '\\leavevmode\\footnotetext[8]{\sphinxAtStartFootnote%\n'
+            'http://sphinx-doc.org/\n}\nDescription' in result)
+    assert ('\\item[{Footnote in term \\protect\\footnotemark[10]}] '
+            '\\leavevmode\\footnotetext[10]{\sphinxAtStartFootnote%\n'
+            'Footnote in term\n}\nDescription' in result)
     assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist}\\protect'
-            '\\footnotemark[8]}] '
-            '\\leavevmode\\footnotetext[8]{\nhttp://sphinx-doc.org/\n}\nDescription' in result)
-    assert ('\\href{https://github.com/sphinx-doc/sphinx}'
-            '{https://github.com/sphinx-doc/sphinx}\n' in result)
+            '\\footnotemark[9]}] '
+            '\\leavevmode\\footnotetext[9]{\sphinxAtStartFootnote%\n'
+            'http://sphinx-doc.org/\n}\nDescription' in result)
+    assert ('\\url{https://github.com/sphinx-doc/sphinx}\n' in result)
     assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
             '{sphinx-dev@googlegroups.com}\n' in result)
 
@@ -407,19 +524,30 @@ def test_latex_show_urls_is_no(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert 'First footnote: \\footnote[2]{\nFirst\n}' in result
-    assert 'Second footnote: \\footnote[1]{\nSecond\n}' in result
+    assert ('Same footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in bar\n} in bar.rst' in result)
+    assert ('Auto footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in baz\n} in baz.rst' in result)
+    assert ('\\phantomsection\\label{index:id26}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to\\string-authoryear]'
+            '{\\crossref{The section with a reference to \\phantomsection\\label{index:id1}'
+            '{\\hyperref[index:authoryear]{\\crossref{{[}AuthorYear{]}}}}}}}' in result)
+    assert ('\\phantomsection\\label{index:id27}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to]'
+            '{\\crossref{The section with a reference to }}}' in result)
+    assert 'First footnote: \\footnote[2]{\sphinxAtStartFootnote%\nFirst\n}' in result
+    assert 'Second footnote: \\footnote[1]{\sphinxAtStartFootnote%\nSecond\n}' in result
     assert '\\href{http://sphinx-doc.org/}{Sphinx}' in result
-    assert 'Third footnote: \\footnote[3]{\nThird\n}' in result
+    assert 'Third footnote: \\footnote[3]{\sphinxAtStartFootnote%\nThird\n}' in result
     assert '\\href{http://sphinx-doc.org/~test/}{URL including tilde}' in result
     assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term}}] '
             '\\leavevmode\nDescription' in result)
     assert ('\\item[{Footnote in term \\protect\\footnotemark[5]}] '
-            '\\leavevmode\\footnotetext[5]{\nFootnote in term\n}\nDescription' in result)
+            '\\leavevmode\\footnotetext[5]{\sphinxAtStartFootnote%\n'
+            'Footnote in term\n}\nDescription' in result)
     assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist}}] '
             '\\leavevmode\nDescription' in result)
-    assert ('\\href{https://github.com/sphinx-doc/sphinx}'
-            '{https://github.com/sphinx-doc/sphinx}\n' in result)
+    assert ('\\url{https://github.com/sphinx-doc/sphinx}\n' in result)
     assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
             '{sphinx-dev@googlegroups.com}\n' in result)
 
@@ -496,3 +624,47 @@ def test_toctree_without_maxdepth(app, status, warning):
     print(status.getvalue())
     print(warning.getvalue())
     assert '\\setcounter{tocdepth}' not in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'latex_toplevel_sectioning': None})
+def test_latex_toplevel_sectioning_is_None(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\chapter{Foo}' in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'latex_toplevel_sectioning': 'part'})
+def test_latex_toplevel_sectioning_is_part(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\part{Foo}' in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'latex_toplevel_sectioning': 'chapter'})
+def test_latex_toplevel_sectioning_is_chapter(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\chapter{Foo}' in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'latex_toplevel_sectioning': 'section'})
+def test_latex_toplevel_sectioning_is_section(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\section{Foo}' in result
